@@ -107,7 +107,7 @@ activate_uri (GtkWidget *menuitem,
 		if (error->code != GNOME_URL_ERROR_CANCELLED) {
 			escaped = g_markup_escape_text (url, -1);
 			panel_error_dialog (screen, "cannot_show_url", TRUE,
-					    _("Cannot display location '%s'"),
+					    _("Could not open location '%s'"),
 					    "%s",
 					    escaped,
 					    error->message);
@@ -304,7 +304,8 @@ panel_place_menu_item_append_gtk_bookmarks (GtkWidget *menu)
 			     !gnome_vfs_uri_exists (uri))) {
 				if (label)
 					g_free (label);
-				gnome_vfs_uri_unref (uri);
+				if (uri)
+					gnome_vfs_uri_unref (uri);
 				continue;
 			}
 
@@ -781,12 +782,16 @@ panel_place_menu_item_instance_init (PanelPlaceMenuItem      *menuitem,
 
 	if (bookmarks_uri) {
 		GnomeVFSResult result;
+
 		result = gnome_vfs_monitor_add (&menuitem->priv->bookmarks_monitor,
 						bookmarks_uri,
 						GNOME_VFS_MONITOR_FILE,
 						panel_place_menu_item_gtk_bookmarks_changed,
 						menuitem);
-		if (result != GNOME_VFS_OK)
+
+		if (result == GNOME_VFS_ERROR_NOT_SUPPORTED)
+			g_message ("File monitoring not supported in the compiled version of gnome-vfs: bookmarks won't be monitored.");
+		else if (result != GNOME_VFS_OK)
 			g_warning ("Failed to add file monitor for %s: %s\n",
 				   bookmarks_uri,
 				   gnome_vfs_result_to_string (result));
@@ -1081,7 +1086,7 @@ panel_menu_item_activate_desktop_file (GtkWidget  *menuitem,
 		if (error) {
 			panel_error_dialog (menuitem_to_screen (menuitem),
 					    "cannot_launch_entry", TRUE,
-					    _("Cannot launch entry"),
+					    _("Could not launch menu item"),
 					    "%s",
 					    error->message);
 
@@ -1093,7 +1098,7 @@ panel_menu_item_activate_desktop_file (GtkWidget  *menuitem,
 
 		panel_error_dialog (menuitem_to_screen (menuitem),
 				    "cannot_load_entry", TRUE,
-				    _("Cannot load entry"),
+				    _("Could not load menu item"),
 				    "%s",
 				    error->message);
 		g_error_free (error);
