@@ -309,16 +309,30 @@ panel_profile_get_queued_changes (GObject *object)
 	return g_object_get_qdata (object, queued_changes_quark);
 }
 
+static void
+panel_porfile_remove_commit_timeout (guint timeout)
+{
+	g_source_remove (timeout);
+}
+
 static void
 panel_profile_set_commit_timeout (PanelToplevel *toplevel,
 				  guint          timeout)
 {
+	GDestroyNotify destroy_notify;
+
 	if (!commit_timeout_quark)
 		commit_timeout_quark = g_quark_from_static_string ("panel-queued-timeout");
 
-	g_object_set_qdata (G_OBJECT (toplevel),
-			    commit_timeout_quark,
-			    GUINT_TO_POINTER (timeout));
+	if (timeout)
+		destroy_notify = (GDestroyNotify) panel_porfile_remove_commit_timeout;
+	else
+		destroy_notify = NULL;
+
+	g_object_set_qdata_full (G_OBJECT (toplevel),
+				 commit_timeout_quark,
+				 GUINT_TO_POINTER (timeout),
+				 destroy_notify);
 }
 
 static guint
